Ext.define('Music.controller.Player', {
	extend: 'Ext.app.Controller',
	stores: ['Songs', 'Playlist'],
	models: ['Song'],
	views: [
		'music.List', 
		'music.Playlist'
	],
	refs: [
		{ref: 'pauseButton', selector: 'playlist button[action=pause]'},
		{ref: 'musiclist', selector: 'musiclist'},
		{ref: 'playlist', selector: 'playlist'}
	],
	
	player: new Audio(),
	
	statusTimer: undefined,
	
	togglePause: function(state) {
		var button = this.getPauseButton();
		button.toggle(state, false);
	},	
	playSong: function(song) {
		if (song.url) {
			// The onended and timeupdate events aren't firing properly on at least chrome
			// and safari on OS X. Work-around is using a timer to poll the player status.
			clearTimeout(this.statusTimer); 
			var that = this;
			this.statusTimer = setTimeout(function () { that.statusHandler(that) }, 1000);
			
			this.togglePause(false);
			this.player.src = song.url;
			this.player.play();
			var playlist = this.getPlaylist();
			playlist.setNowPlaying(song.name + ' - ' + song.artist);

			// Highlight the currently playing item.
			// Deactivate all other songs.
			this.getPlaylistStore().each(function(record) { record.data.active = false; });
			song.active = true;
			this.getPlaylist().getView().refresh();
		}
	},
	getCurrentSong: function() {
		var currentSong;
		this.getPlaylistStore().each(function(record) {
			if (record.data.active) {
				currentSong = record;
				return false;
			}
		});
		return currentSong;
	},
	statusHandler: function(that) {
		var plStore = this.getPlaylistStore();
		if (this.player.ended) {
			// Find the song that was active when the player ended.
			var currentSong = this.getCurrentSong();

			// Get the next song in the playlist and play it.
			if (currentSong) {
				var index = plStore.indexOf(currentSong);
				var nextSong = plStore.getAt(index+1);
				if (nextSong) {
					this.playSong(nextSong.data);
				} else {
					this.getPlaylist().setNowPlaying('');
					this.getPlaylist().updateProgress(0, 0);
					currentSong.data.active = false;
					this.getPlaylist().getView().refresh();	
				}	
			}				
		} else {
			this.statusTimer = setTimeout(function () { that.statusHandler(that) }, 1000);
		}

		// Update progress.
		if (this.isPlaying()) {
			this.getPlaylist().updateProgress(this.player.currentTime, this.player.duration);
		}
	},
	init: function() {
		this.control({
			'musiclist': {
				itemdblclick: this.songDblClick
			},
			'musiclist button[action=addToPlaylist]': {
				click: this.addToPlaylist
			},
			'musiclist button[action=download]': {
				click: this.downloadButton
			},
			'playlist': {
				itemdblclick: this.playlistDblClick,
				render: function(view) {
					view.setVolumeSlider(this.player.volume * 100);
				} 
			},
			'playlist button[action=play]': {
				click: this.playSongButton
			},
			'playlist button[action=pause]': {
				click: this.pauseSongButton
			},
			'playlist button[action=prev]': {
				click: this.prevSongButton
			},
			'playlist button[action=next]': {
				click: this.nextSongButton
			},
			'playlist button[action=removeFromPlaylist]': {
				click: this.removeFromPlaylist
			},
			'playlist button[action=clearPlaylist]': {
				click: this.clearPlaylist
			},
			'playlist slider[itemId=progressSlider]': {
				changecomplete: this.progressSliderChanged,
				change: this.progressSliderChange
			},
			'playlist slider[itemId=volumeSlider]': {
				change: this.volumeSliderChange
			},
			'playlist button[action=songmeanings]': {
				click: this.googleSongMeanings
			}
		});
	},
	songDblClick: function(view, record) {
		if (record.data.leaf) {
			var store = this.getPlaylistStore();
			var index = store.getCount();
			store.insert(index, record.data);
			if (!this.isPlaying()) {
				var newRecord = store.getAt(index);
				this.playSong(newRecord.data);
			}
		}
	},
	addToPlaylist: function(button) {
		var store = this.getPlaylistStore();
		var count = store.getCount();
		var nodes = this.getMusiclist().getView().getSelectedNodes();
		for (var node in nodes) {
			var record = this.getMusiclist().getView().getRecord(nodes[node]);
			if (record.data.leaf) {
				store.insert(count++, record.data);
			}
		}
	},
	downloadButton: function(button) {
		// Just download the first one if several are selected.
		var nodes = this.getMusiclist().getView().getSelectedNodes();
		if (nodes.length > 0) {
			var record = this.getMusiclist().getView().getRecord(nodes[0]);
			window.open(record.data.url, "Download");
		}
	},
	playlistDblClick: function(view, record) {
		this.playSong(record.data);
	},
	playSongButton: function(button) {
		var playlistView = this.getPlaylist().getView();
		var selection = playlistView.getSelectedNodes();
		if (selection) {
			var song = playlistView.getRecord(selection[0]).data;
			this.playSong(song);
		}
	},
	pauseSongButton: function(button) {
		if (button.pressed) {
			this.player.pause();
		} else {
			this.player.play();
		}
	},
	prevSongButton: function(button) {
		var currentIndex = this.getPlaylistStore().indexOf(this.getCurrentSong());
		var prevSong = this.getPlaylistStore().getAt(currentIndex-1);
		if (prevSong) {
			this.playSong(prevSong.data);
		}
	},
	nextSongButton: function(button) {
		var currentIndex = this.getPlaylistStore().indexOf(this.getCurrentSong());
		var nextSong = this.getPlaylistStore().getAt(currentIndex+1);
		if (nextSong) {
			this.playSong(nextSong.data);
		}
	},
	clearPlaylist: function(button) {
		this.getPlaylistStore().removeAll();
		this.player.pause();
		this.getPlaylist().setNowPlaying('');
		this.getPlaylist().updateProgress(0, 0);
	},
	removeFromPlaylist: function(button) {
		var songs = this.getSelectedPlaylistSongs();
		for (var s in songs) {
			var index = this.getPlaylistStore().indexOf(songs[s]);
			this.getPlaylistStore().removeAt(index)
		}
	},
	progressSliderChanged: function(slider, newValue) {
		if (this.player.readyState != this.player.HAVE_NOTHING) {
			var progress = newValue / 100;
            this.player.currentTime = progress * this.player.duration;
		}
	},
	progressSliderChange: function(slider, newValue) {
		var progress = newValue / 100;
		this.getPlaylist().setProgressText(progress * this.player.duration, this.player.duration);
	},
	volumeSliderChange: function(slider, newValue) {
		this.player.volume = newValue / 100;
	},
	googleSongMeanings: function() {
		var selection = this.getSelectedPlaylistSongs();
		if (selection && selection.length == 1) {
			selection[0].googleSongMeanings();
		}
	},

	// Utility functions.
	getSelectedPlaylistSongs: function() {
		var playlistView = this.getPlaylist().getView();
		var selection = playlistView.getSelectedNodes();
		if (selection) {
			return playlistView.getRecords(selection);
		}
	},

	isPlaying: function() {
		return !this.player.paused && !this.player.ended;
	}
});
